from __future__ import annotations

import collections.abc
import types
import typing

import amulet.core.selection.box

__all__ = ["SelectionGroup"]

class SelectionGroup:
    """
    A container for zero or more :class:`SelectionBox` instances.

    This allows for non-rectangular and non-contiguous selections.
    """

    __hash__: typing.ClassVar[None] = None  # type: ignore
    def __bool__(self) -> bool:
        """
        The number of :class:`SelectionBox` classes in the group.
        """

    @typing.overload
    def __eq__(self, arg0: SelectionGroup) -> bool:
        """
        Does the contents of this :class:`SelectionGroup` match the other :class:`SelectionGroup`.

        Note if the boxes do not exactly match this will return False even if the volume represented is the same.

        :param other: The other :class:`SelectionGroup` to compare with.
        :return: True if the boxes contained match.
        """

    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    @typing.overload
    def __init__(self) -> None:
        """
        Create an empty SelectionGroup.

        >>> SelectionGroup()
        """

    @typing.overload
    def __init__(self, box: amulet.core.selection.box.SelectionBox) -> None:
        """
        Create a SelectionGroup containing the given box.

        >>> SelectionGroup(SelectionBox(0, 0, 0, 1, 1, 1))
        """

    @typing.overload
    def __init__(
        self, boxes: collections.abc.Iterable[amulet.core.selection.box.SelectionBox]
    ) -> None:
        """
        Create a SelectionGroup from the boxes in the iterable.

        >>> SelectionGroup([
        >>>     SelectionBox(0, 0, 0, 1, 1, 1),
        >>>     SelectionBox(1, 1, 1, 1, 1, 1)
        >>> ])
        """

    def __iter__(
        self,
    ) -> collections.abc.Iterator[amulet.core.selection.box.SelectionBox]:
        """
        An iterable of all the :class:`SelectionBox` classes in the group.
        """

    def __len__(self) -> int:
        """
        The number of :class:`SelectionBox` classes in the group.
        """

    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def contains_block(
        self, x: typing.SupportsInt, y: typing.SupportsInt, z: typing.SupportsInt
    ) -> bool:
        """
        Is the block contained within the selection.

        >>> selection1: AbstractBaseSelection
        >>> (1, 2, 3) in selection1
        True

        :param x: The x coordinate of the block. Defined by the most negative corner.
        :param y: The y coordinate of the block. Defined by the most negative corner.
        :param z: The z coordinate of the block. Defined by the most negative corner.
        :return: True if the block is in the selection.
        """

    def contains_point(
        self, x: typing.SupportsFloat, y: typing.SupportsFloat, z: typing.SupportsFloat
    ) -> bool:
        """
        Is the point contained within the selection.

        >>> selection1: AbstractBaseSelection
        >>> (1.5, 2.5, 3.5) in selection1
        True

        :param x: The x coordinate of the point.
        :param y: The y coordinate of the point.
        :param z: The z coordinate of the point.
        :return: True if the point is in the selection.
        """

    @typing.overload
    def intersects(self, other: amulet.core.selection.box.SelectionBox) -> bool:
        """
        Does this selection intersect ``other``.

        :param other: The other selection.
        :return: True if the selections intersect, False otherwise.
        """

    @typing.overload
    def intersects(self, other: SelectionGroup) -> bool: ...
    def translate(
        self, x: typing.SupportsInt, y: typing.SupportsInt, z: typing.SupportsInt
    ) -> SelectionGroup:
        """
        Create a new :class:`SelectionGroup` based on this one with the coordinates moved by the given offset.

        :param x: The x offset.
        :param y: The y offset.
        :param z: The z offset.
        :return: The new selection with the given offset.
        """

    @property
    def bounding_box(self) -> amulet.core.selection.box.SelectionBox:
        """
        A SelectionBox containing this entire selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def bounds(self) -> tuple[tuple[int, int, int], tuple[int, int, int]]:
        """
        The minimum and maximum x, y and z coordinates in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def max(self) -> tuple[int, int, int]:
        """
        The maximum x, y and z coordinates in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def max_x(self) -> int:
        """
        The maximum x coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def max_y(self) -> int:
        """
        The maximum y coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def max_z(self) -> int:
        """
        The maximum z coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def min(self) -> tuple[int, int, int]:
        """
        The minimum x, y and z coordinates in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def min_x(self) -> int:
        """
        The minimum x coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def min_y(self) -> int:
        """
        The minimum y coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def min_z(self) -> int:
        """
        The minimum z coordinate in the selection.

        :raises RuntimeError: If there are no boxes in the selection.
        """

    @property
    def selection_boxes(
        self,
    ) -> collections.abc.Iterator[amulet.core.selection.box.SelectionBox]:
        """
        An iterator of the :class:`SelectionBox` instances stored for this group.
        """



from __future__ import annotations

import types
import typing

import amulet.core.selection.group

__all__ = ["SelectionBox"]

class SelectionBox:
    """
    The SelectionBox class represents a single cuboid selection.

    When combined with :class:`~amulet.api.selection.SelectionGroup` it can represent any arbitrary shape.
    """

    @typing.overload
    def __eq__(self, arg0: SelectionBox) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __ge__(self, arg0: SelectionBox) -> bool: ...
    def __gt__(self, arg0: SelectionBox) -> bool: ...
    def __hash__(self) -> int: ...
    @typing.overload
    def __init__(
        self,
        min_x: typing.SupportsInt,
        min_y: typing.SupportsInt,
        min_z: typing.SupportsInt,
        size_x: typing.SupportsInt,
        size_y: typing.SupportsInt,
        size_z: typing.SupportsInt,
    ) -> None:
        """
        Construct a new SelectionBox instance.

        >>> # a selection box that selects one block.
        >>> box = SelectionBox(0, 0, 0, 1, 1, 1)

        :param min_x: The minimum x coordinate of the box.
        :param min_y: The minimum y coordinate of the box.
        :param min_z: The minimum z coordinate of the box.
        :param size_x: The size of the box in the x axis.
        :param size_y: The size of the box in the y axis.
        :param size_z: The size of the box in the z axis.
        """

    @typing.overload
    def __init__(
        self,
        point_1: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt],
        point_2: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt],
    ) -> None:
        """
        Construct a new SelectionBox instance.

        >>> # a selection box that selects one block.
        >>> box = SelectionBox((0, 0, 0), (1, 1, 1))

        :param point_1: The first coordinate of the box.
        :param point_2: The second coordinate of the box.
        """

    def __le__(self, arg0: SelectionBox) -> bool: ...
    def __lt__(self, arg0: SelectionBox) -> bool: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def contains_block(
        self, x: typing.SupportsInt, y: typing.SupportsInt, z: typing.SupportsInt
    ) -> bool:
        """
        Is the block contained within the selection.

        >>> selection1: AbstractBaseSelection
        >>> (1, 2, 3) in selection1
        True

        :param x: The x coordinate of the block. Defined by the most negative corner.
        :param y: The y coordinate of the block. Defined by the most negative corner.
        :param z: The z coordinate of the block. Defined by the most negative corner.
        :return: True if the block is in the selection.
        """

    def contains_box(self, other: SelectionBox) -> bool:
        """
        Does the other SelectionBox other fit entirely within this SelectionBox.

        :param other: The SelectionBox to test.
        :return: True if other fits in self, False otherwise.
        """

    def contains_point(
        self, x: typing.SupportsFloat, y: typing.SupportsFloat, z: typing.SupportsFloat
    ) -> bool:
        """
        Is the point contained within the selection.

        >>> selection1: AbstractBaseSelection
        >>> (1.5, 2.5, 3.5) in selection1
        True

        :param x: The x coordinate of the point.
        :param y: The y coordinate of the point.
        :param z: The z coordinate of the point.
        :return: True if the point is in the selection.
        """

    @typing.overload
    def intersects(self, other: SelectionBox) -> bool:
        """
        Does this selection intersect ``other``.

        :param other: The other selection.
        :return: True if the selections intersect, False otherwise.
        """

    @typing.overload
    def intersects(self, other: amulet.core.selection.group.SelectionGroup) -> bool: ...
    def touches(self, other: SelectionBox) -> bool:
        """
        Method to check if this instance of :class:`SelectionBox` touches but does not intersect another SelectionBox.

        :param other: The other SelectionBox
        :return: True if the two :class:`SelectionBox` instances touch, False otherwise
        """

    def touches_or_intersects(self, other: SelectionBox) -> bool:
        """
        Does this SelectionBox touch or intersect the other SelectionBox.

        :param other: The other SelectionBox.
        :return: True if the two :class:`SelectionBox` instances touch or intersect, False otherwise.
        """

    def translate(
        self, x: typing.SupportsInt, y: typing.SupportsInt, z: typing.SupportsInt
    ) -> SelectionBox:
        """
        Create a new :class:`SelectionBox` based on this one with the coordinates moved by the given offset.

        :param x: The x offset.
        :param y: The y offset.
        :param z: The z offset.
        :return: The new selection with the given offset.
        """

    @property
    def max(self) -> tuple[int, int, int]:
        """
        The maximum coordinate of the box.
        """

    @property
    def max_x(self) -> int:
        """
        The maximum x coordinate of the box.
        """

    @property
    def max_y(self) -> int:
        """
        The maximum y coordinate of the box.
        """

    @property
    def max_z(self) -> int:
        """
        The maximum z coordinate of the box.
        """

    @property
    def min(self) -> tuple[int, int, int]:
        """
        The minimum coordinate of the box.
        """

    @property
    def min_x(self) -> int:
        """
        The minimum x coordinate of the box.
        """

    @property
    def min_y(self) -> int:
        """
        The minimum y coordinate of the box.
        """

    @property
    def min_z(self) -> int:
        """
        The minimum z coordinate of the box.
        """

    @property
    def shape(self) -> tuple[int, int, int]:
        """
        The length of the box in the x, y and z axis.

        >>> SelectionBox(0, 0, 0, 1, 1, 1).shape
        (1, 1, 1)
        """

    @property
    def size_x(self) -> int:
        """
        The length of the box in the x axis.
        """

    @property
    def size_y(self) -> int:
        """
        The length of the box in the y axis.
        """

    @property
    def size_z(self) -> int:
        """
        The length of the box in the z axis.
        """

    @property
    def volume(self) -> int:
        """
        The number of blocks in the box.

        >>> SelectionBox(0, 0, 0, 1, 1, 1).volume
        1
        """



from __future__ import annotations

from amulet.core.chunk.component.block_component import (
    BlockComponent,
    BlockComponentData,
)
from amulet.core.chunk.component.section_array_map import IndexArray3D, SectionArrayMap

from . import block_component, section_array_map

__all__ = [
    "BlockComponent",
    "BlockComponentData",
    "IndexArray3D",
    "SectionArrayMap",
    "block_component",
    "section_array_map",
]



from __future__ import annotations

from . import (
    _amulet_core,
    _version,
    biome,
    block,
    block_entity,
    chunk,
    entity,
    palette,
    selection,
    version,
)

__all__ = [
    "biome",
    "block",
    "block_entity",
    "chunk",
    "compiler_config",
    "entity",
    "palette",
    "selection",
    "version",
]

def _init() -> None: ...

__version__: str
compiler_config: dict



from __future__ import annotations

import collections.abc
import types
import typing

import numpy
import numpy.typing

__all__ = ["IndexArray3D", "SectionArrayMap"]

class IndexArray3D:
    """
    A 3D index array.
    """

    @typing.overload
    def __init__(
        self, shape: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt]
    ) -> None: ...
    @typing.overload
    def __init__(
        self,
        shape: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt],
        value: typing.SupportsInt,
    ) -> None: ...
    @typing.overload
    def __init__(self, other: IndexArray3D) -> None: ...
    @typing.overload
    def __init__(self, arg0: collections.abc.Buffer) -> None: ...
    @property
    def shape(self) -> tuple[int, int, int]: ...
    @property
    def size(self) -> int: ...

class SectionArrayMap:
    """
    A container of sub-chunk arrays.
    """

    def __contains__(self, arg0: typing.SupportsInt) -> bool: ...
    def __delitem__(self, arg0: typing.SupportsInt) -> None: ...
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __getitem__(
        self, arg0: typing.SupportsInt
    ) -> numpy.typing.NDArray[numpy.uint32]: ...
    def __hash__(self) -> int: ...
    def __init__(
        self,
        array_shape: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt],
        default_array: (
            typing.SupportsInt
            | amulet.core.chunk.component.section_array_map.IndexArray3D
            | collections.abc.Buffer
        ),
    ) -> None: ...
    def __iter__(self) -> collections.abc.Iterator[int]: ...
    def __len__(self) -> int: ...
    def __setitem__(
        self,
        arg0: typing.SupportsInt,
        arg1: (
            amulet.core.chunk.component.section_array_map.IndexArray3D
            | collections.abc.Buffer
        ),
    ) -> None: ...
    def get(
        self,
        key: typing.SupportsInt,
        default: numpy.typing.NDArray[numpy.uint32] | None = None,
    ) -> numpy.typing.NDArray[numpy.uint32] | None: ...
    def items(
        self,
    ) -> collections.abc.ItemsView[int, numpy.typing.NDArray[numpy.uint32]]: ...
    def keys(self) -> collections.abc.KeysView[int]: ...
    def pop(
        self, key: typing.SupportsInt, default: numpy.typing.NDArray[numpy.uint32] = ...
    ) -> numpy.typing.NDArray[numpy.uint32]: ...
    def popitem(self) -> tuple[int, numpy.typing.NDArray[numpy.uint32]]: ...
    def populate(self, arg0: typing.SupportsInt) -> None: ...
    def setdefault(
        self, arg0: typing.SupportsInt, arg1: numpy.typing.NDArray[numpy.uint32] | None
    ) -> numpy.typing.NDArray[numpy.uint32] | None: ...
    def update(self, other: typing.Any = (), **kwargs: typing.Any) -> None: ...
    def values(
        self,
    ) -> collections.abc.ValuesView[numpy.typing.NDArray[numpy.uint32]]: ...
    @property
    def array_shape(self) -> tuple[int, int, int]: ...
    @property
    def default_array(self) -> int | numpy.ndarray: ...
    @default_array.setter
    def default_array(
        self,
        arg1: (
            typing.SupportsInt
            | amulet.core.chunk.component.section_array_map.IndexArray3D
            | collections.abc.Buffer
        ),
    ) -> None: ...



from __future__ import annotations

import collections.abc
import typing

import amulet.core.biome
import amulet.core.version

__all__ = ["BiomePalette"]

class BiomePalette(amulet.core.version.VersionRangeContainer):
    @typing.overload
    def __contains__(self, arg0: typing.SupportsInt) -> bool: ...
    @typing.overload
    def __contains__(self, arg0: amulet.core.biome.Biome) -> bool: ...
    @typing.overload
    def __getitem__(self, arg0: typing.SupportsInt) -> amulet.core.biome.Biome: ...
    @typing.overload
    def __getitem__(self, arg0: slice) -> list: ...
    def __init__(self, arg0: amulet.core.version.VersionRange) -> None: ...
    def __iter__(self) -> collections.abc.Iterator[typing.Any]: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __reversed__(self) -> collections.abc.Iterator[typing.Any]: ...
    def biome_to_index(self, arg0: amulet.core.biome.Biome) -> int:
        """
        Get the index of the biome in the palette.
        If it is not in the palette already it will be added first.

        :param biome: The biome to get the index of.
        :return: The index of the biome in the palette.
        """

    def count(self, value: typing.Any) -> int: ...
    def index(
        self,
        value: typing.Any,
        start: typing.SupportsInt = 0,
        stop: typing.SupportsInt = 9223372036854775807,
    ) -> int: ...
    def index_to_biome(self, arg0: typing.SupportsInt) -> amulet.core.biome.Biome:
        """
        Get the biome at the specified palette index.

        :param index: The index to get
        :return: The biome at that index
        :raises IndexError if there is no biome at that index.
        """



from __future__ import annotations

import types
import typing

import amulet.core.version

__all__ = ["Biome"]

class Biome(amulet.core.version.PlatformVersionContainer):
    """
    A class to manage the state of a biome.

    It is an immutable object that contains the platform, version, namespace and base name.

    Here's a few examples on how create a Biome object:

    >>> # Create a plains biome for Java 1.20.2
    >>> plains = Biome("java", VersionNumber(3578), "minecraft", "plains")
    >>> # The version number for Java is the Java data version

    >>> # Create a plains biome for Bedrock
    >>> plains = Biome("bedrock", VersionNumber(1, 21, 0, 3), "minecraft", "plains")
    >>> # The biome version number is unused in Bedrock but it is here for completeness.
    """

    @typing.overload
    def __eq__(self, arg0: Biome) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __ge__(self, arg0: Biome) -> bool: ...
    def __gt__(self, arg0: Biome) -> bool: ...
    def __hash__(self) -> int: ...
    def __init__(
        self,
        platform: str,
        version: amulet.core.version.VersionNumber,
        namespace: str,
        base_name: str,
    ) -> None: ...
    def __le__(self, arg0: Biome) -> bool: ...
    def __lt__(self, arg0: Biome) -> bool: ...
    def __repr__(self) -> str: ...
    @property
    def base_name(self) -> str:
        """
        The base name of the :class:`Biome` object.

        >>> biome: Biome
        >>> biome.base_name

        :return: The base name of the biome
        """

    @property
    def namespace(self) -> str:
        """
        The namespace of the :class:`Biome` object.

        >>> biome: Biome
        >>> water.namespace

        :return: The namespace of the biome
        """

    @property
    def namespaced_name(self) -> str:
        """
        The namespaced id of the :class:`Biome` object.

        >>> biome: Biome
        >>> biome.namespaced_name

        :return: The "namespace:base_name" of the biome
        """



from __future__ import annotations

import collections.abc
import types
import typing

import amulet.core.version
import amulet.nbt

__all__ = ["Block", "BlockStack"]

class Block(amulet.core.version.PlatformVersionContainer):
    """
    A class to manage the state of a block.

    It is an immutable object that contains the platform, version, namespace, base name and properties.

    Here's a few examples on how create a Block object:

    >>> # Create a stone block for Java 1.20.2
    >>> stone = Block("java", VersionNumber(3578), "minecraft", "stone")
    >>> # The Java block version number is the Java data version

    >>> # Create a stone block for Bedrock
    >>> stone = Block("bedrock", VersionNumber(1, 21, 0, 3), "minecraft", "stone")
    >>> # The Bedrock block version number is the value stored as an int with the block data.

    >>> # Create a Java water block with the level property
    >>> water = Block(
    >>>     "java", VersionNumber(3578),
    >>>     "minecraft",  # the namespace
    >>>     "water",  # the base name
    >>>     {  # A dictionary of properties.
    >>>         # Keys must be strings and values must be a numerical or string NBT type.
    >>>         "level": StringTag("0")  # define a property `level` with a string value `0`
    >>>     }
    >>> )
    """

    PropertyValue: typing.ClassVar[typing.TypeAlias] = (
        amulet.nbt.ByteTag
        | amulet.nbt.ShortTag
        | amulet.nbt.IntTag
        | amulet.nbt.LongTag
        | amulet.nbt.StringTag
    )
    @staticmethod
    def from_bedrock_blockstate(
        platform: str, version: amulet.core.version.VersionNumber, blockstate: str
    ) -> Block:
        """
        Parse a Bedrock format blockstate where values are all strings and populate a :class:`Block` class with the data.

        >>> stone = Block.from_bedrock_blockstate("minecraft:stone")
        >>> water = Block.from_bedrock_blockstate("minecraft:water["liquid_depth"=0]")

        :param platform: The platform the block is defined in.
        :param version: The version the block is defined in.
        :param blockstate: The Bedrock blockstate string to parse.
        :return: A Block instance containing the state.
        """

    @staticmethod
    def from_java_blockstate(
        platform: str, version: amulet.core.version.VersionNumber, blockstate: str
    ) -> Block:
        """
        Parse a Java format blockstate where values are all strings and populate a :class:`Block` class with the data.

        >>> stone = Block.from_java_blockstate("minecraft:stone")
        >>> water = Block.from_java_blockstate("minecraft:water[level=0]")

        :param platform: The platform the block is defined in.
        :param version: The version the block is defined in.
        :param blockstate: The Java blockstate string to parse.
        :return: A Block instance containing the state.
        """

    @typing.overload
    def __eq__(self, arg0: Block) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __ge__(self, arg0: Block) -> bool: ...
    def __gt__(self, arg0: Block) -> bool: ...
    def __hash__(self) -> int: ...
    def __init__(
        self,
        platform: str,
        version: amulet.core.version.VersionNumber,
        namespace: str,
        base_name: str,
        properties: collections.abc.Mapping[
            str,
            amulet.nbt.ByteTag
            | amulet.nbt.ShortTag
            | amulet.nbt.IntTag
            | amulet.nbt.LongTag
            | amulet.nbt.StringTag,
        ] = {},
    ) -> None: ...
    def __le__(self, arg0: Block) -> bool: ...
    def __lt__(self, arg0: Block) -> bool: ...
    def __repr__(self) -> str: ...
    @property
    def base_name(self) -> str:
        """
        The base name of the blockstate represented by the :class:`Block` object.

        >>> block: Block
        >>> block.base_name

        :return: The base name of the blockstate
        """

    @property
    def bedrock_blockstate(self) -> str:
        """
        The Bedrock blockstate string of this :class:`Block` object.
        Converts the property values to the SNBT format to preserve type.

        >>> bell = Block(
        >>>     "java", VersionNumber(3578),
        >>>     "minecraft",
        >>>     "bell",
        >>>     {
        >>>         "attachment":StringTag("standing"),
        >>>         "direction":IntTag(0),
        >>>         "toggle_bit":ByteTag(0)
        >>>     }
        >>> )
        >>> bell.bedrock_blockstate
        minecraft:bell["attachment"="standing","direction"=0,"toggle_bit"=false]

        :return: The SNBT blockstate string
        """

    @property
    def java_blockstate(self) -> str:
        """
        The Java blockstate string of this :class:`Block` object.
        Note this will only contain properties with StringTag values.

        >>> stone = Block("java", VersionNumber(3578), "minecraft", "stone")
        >>> stone.java_blockstate
        minecraft:stone
        >>> water = Block("java", VersionNumber(3578), "minecraft", "water", {"level": StringTag("0")})
        >>> water.java_blockstate
        minecraft:water[level=0]

        :return: The blockstate string
        """

    @property
    def namespace(self) -> str:
        """
        The namespace of the blockstate represented by the :class:`Block` object.

        >>> block: Block
        >>> water.namespace

        :return: The namespace of the blockstate
        """

    @property
    def namespaced_name(self) -> str:
        """
        The namespace:base_name of the blockstate represented by the :class:`Block` object.

        >>> block: Block
        >>> block.namespaced_name

        :return: The namespace:base_name of the blockstate
        """

    @property
    def properties(
        self,
    ) -> dict[
        str,
        amulet.nbt.ByteTag
        | amulet.nbt.ShortTag
        | amulet.nbt.IntTag
        | amulet.nbt.LongTag
        | amulet.nbt.StringTag,
    ]:
        """
        The properties of the blockstate represented by the :class:`Block` object as a dictionary.
        >>> block: Block
        >>> block.properties

        :return: A mapping of the properties of the blockstate
        """

class BlockStack:
    """
    A stack of block objects.

    Java 1.13 added the concept of waterlogging blocks whereby some blocks have a `waterlogged` property.
    Bedrock achieved the same behaviour by added a layering system which allows the second block to be any block.

    Amulet supports both implementations with a stack of one or more block objects similar to how Bedrock handles it.
    Amulet places no restrictions on which blocks can be extra blocks.
    Extra block may be discarded if the format does not support them.

    Create a waterlogged stone block.
    >>> waterlogged_stone = BlockStack(
    >>>     Block("java", VersionNumber(3578), "minecraft", "stone"),
    >>>     Block("java", VersionNumber(3578), "minecraft", "water", {"level": StringTag("0")})
    >>> )

    Get a block at an index
    >>> stone = waterlogged_stone[0]
    >>> water = waterlogged_stone[1]

    Get the blocks as a list
    >>> blocks = list(waterlogged_stone)
    """

    def __contains__(self, arg0: typing.Any) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: BlockStack) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __ge__(self, arg0: BlockStack) -> bool: ...
    @typing.overload
    def __getitem__(self, arg0: typing.SupportsInt) -> Block: ...
    @typing.overload
    def __getitem__(self, arg0: slice) -> list: ...
    def __gt__(self, arg0: BlockStack) -> bool: ...
    def __hash__(self) -> int: ...
    def __init__(self, block: Block, *extra_blocks: Block) -> None: ...
    def __iter__(self) -> collections.abc.Iterator[Block]: ...
    def __le__(self, arg0: BlockStack) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, arg0: BlockStack) -> bool: ...
    def __repr__(self) -> str: ...
    def __reversed__(self) -> collections.abc.Iterator[Block]: ...
    def count(self, value: typing.Any) -> int: ...
    def index(
        self,
        value: typing.Any,
        start: typing.SupportsInt = 0,
        stop: typing.SupportsInt = 9223372036854775807,
    ) -> int: ...
    @property
    def base_block(self) -> Block:
        """
        The first block in the stack.

        >>> waterlogged_stone = BlockStack(
        >>>     Block("java", VersionNumber(3578), "minecraft", "stone"),
        >>>     Block("java", VersionNumber(3578), "minecraft", "water", {"level": StringTag("0")})
        >>> )
        >>> waterlogged_stone.base_block
        Block("java", VersionNumber(3578), "minecraft", "stone")

        :return: A Block object
        """

    @property
    def extra_blocks(self) -> tuple[Block]:
        """
        The extra blocks in the stack.

        >>> waterlogged_stone = BlockStack(
        >>>     Block("java", VersionNumber(3578), "minecraft", "stone"),
        >>>     Block("java", VersionNumber(3578), "minecraft", "water", {"level": StringTag("0")})
        >>> )
        >>> waterlogged_stone.extra_blocks
        (Block("java", VersionNumber(3578), "minecraft", "water", {"level": StringTag("0")}),)

        :return: A tuple of :class:`Block` objects
        """



from __future__ import annotations

import typing

import amulet.core.block
import amulet.core.chunk.component.section_array_map
import amulet.core.palette.block_palette
import amulet.core.version

__all__ = ["BlockComponent", "BlockComponentData"]

class BlockComponent:
    ComponentID: typing.ClassVar[str] = "Amulet::BlockComponent"
    block: BlockComponentData

class BlockComponentData:
    def __init__(
        self,
        version_range: amulet.core.version.VersionRange,
        array_shape: tuple[typing.SupportsInt, typing.SupportsInt, typing.SupportsInt],
        default_block: amulet.core.block.BlockStack,
    ) -> None: ...
    @property
    def palette(self) -> amulet.core.palette.block_palette.BlockPalette: ...
    @property
    def sections(
        self,
    ) -> amulet.core.chunk.component.section_array_map.SectionArrayMap: ...



from __future__ import annotations

import types
import typing

import amulet.core.version
import amulet.nbt

__all__ = ["Entity"]

class Entity(amulet.core.version.PlatformVersionContainer):
    """
    A class to contain all the data to define an Entity.
    """

    @typing.overload
    def __eq__(self, arg0: Entity) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __hash__(self) -> int: ...
    def __init__(
        self,
        platform: str,
        version: amulet.core.version.VersionNumber,
        namespace: str,
        base_name: str,
        x: typing.SupportsFloat,
        y: typing.SupportsFloat,
        z: typing.SupportsFloat,
        nbt: amulet.nbt.NamedTag,
    ) -> None: ...
    def __repr__(self) -> str: ...
    @property
    def base_name(self) -> str:
        """
        The base name of the entity represented by the :class:`Entity` object.

        >>> entity: Entity
        >>> entity.base_name

        :return: The base name of the entity
        """

    @base_name.setter
    def base_name(self, arg1: str) -> None: ...
    @property
    def namespace(self) -> str:
        """
        The namespace of the entity represented by the :class:`Entity` object.

        >>> entity: Entity
        >>> entity.namespace

        :return: The namespace of the entity
        """

    @namespace.setter
    def namespace(self, arg1: str) -> None: ...
    @property
    def namespaced_name(self) -> str:
        """
        The namespace:base_name of the entity represented by the :class:`Entity` object.

        >>> entity: Entity
        >>> entity.namespaced_name

        :return: The namespace:base_name of the entity
        """

    @property
    def nbt(self) -> amulet.nbt.NamedTag:
        """
        The nbt data for the entity.
        >>> entity: Entity
        >>> entity.nbt

        :return: The NamedTag of the entity
        """

    @nbt.setter
    def nbt(self, arg1: amulet.nbt.NamedTag) -> None: ...
    @property
    def x(self) -> float:
        """
        The x coordinate of the entity.
        """

    @x.setter
    def x(self, arg1: typing.SupportsFloat) -> None: ...
    @property
    def y(self) -> float:
        """
        The y coordinate of the entity.
        """

    @y.setter
    def y(self, arg1: typing.SupportsFloat) -> None: ...
    @property
    def z(self) -> float:
        """
        The z coordinate of the entity.
        """

    @z.setter
    def z(self, arg1: typing.SupportsFloat) -> None: ...



from __future__ import annotations

from amulet.core.palette.biome_palette import BiomePalette
from amulet.core.palette.block_palette import BlockPalette

from . import biome_palette, block_palette

__all__ = ["BiomePalette", "BlockPalette", "biome_palette", "block_palette"]



from __future__ import annotations

from amulet.core.selection.box import SelectionBox
from amulet.core.selection.group import SelectionGroup

from . import box, group

__all__ = ["SelectionBox", "SelectionGroup", "box", "group"]



from __future__ import annotations

from . import component

__all__ = [
    "Chunk",
    "ChunkDoesNotExist",
    "ChunkLoadError",
    "component",
    "get_null_chunk",
]

class Chunk:
    """
    A base class for all chunk classes.
    """

    def reconstruct_chunk(self, arg0: dict[str, bytes | None]) -> None:
        """
        This is private. Do not use this. It will be removed in the future.
        """

    def serialise_chunk(self) -> dict[str, bytes | None]:
        """
        This is private. Do not use this. It will be removed in the future.
        """

    @property
    def chunk_id(self) -> str: ...
    @property
    def component_ids(self) -> set[str]: ...

class ChunkDoesNotExist(ChunkLoadError):
    """
    An error thrown if a chunk does not exist and therefor cannot be loaded.

    >>> try:
    >>>     # get chunk
    >>>     chunk = world.get_chunk(cx, cz, dimension)
    >>> except ChunkDoesNotExist:
    >>>     # will catch all chunks that do not exist
    >>>     # will not catch corrupt chunks
    >>> except ChunkLoadError:
    >>>     # will only catch chunks that errored during loading
    >>>     # chunks that do not exist were caught by the previous except section.
    """

class ChunkLoadError(RuntimeError):
    """
    An error thrown if a chunk failed to load for some reason.

    This may be due to a corrupt chunk, an unsupported chunk format or just because the chunk does not exist to be loaded.

    Catching this error will also catch :class:`ChunkDoesNotExist`

    >>> try:
    >>>     # get chunk
    >>>     chunk = world.get_chunk(cx, cz, dimension)
    >>> except ChunkLoadError:
    >>>     # will catch all chunks that have failed to load
    >>>     # either because they do not exist or errored during loading.
    """

def get_null_chunk(arg0: str) -> Chunk:
    """
    This is a private function
    """



from __future__ import annotations

import collections.abc
import typing

import amulet.core.block
import amulet.core.version

__all__ = ["BlockPalette"]

class BlockPalette(amulet.core.version.VersionRangeContainer):
    @typing.overload
    def __contains__(self, arg0: typing.SupportsInt) -> bool: ...
    @typing.overload
    def __contains__(self, arg0: amulet.core.block.BlockStack) -> bool: ...
    @typing.overload
    def __getitem__(self, arg0: typing.SupportsInt) -> amulet.core.block.BlockStack: ...
    @typing.overload
    def __getitem__(self, arg0: slice) -> list: ...
    def __init__(self, arg0: amulet.core.version.VersionRange) -> None: ...
    def __iter__(self) -> collections.abc.Iterator[amulet.core.block.BlockStack]: ...
    def __len__(self) -> int: ...
    def __repr__(self) -> str: ...
    def __reversed__(
        self,
    ) -> collections.abc.Iterator[amulet.core.block.BlockStack]: ...
    def block_stack_to_index(self, arg0: amulet.core.block.BlockStack) -> int:
        """
        Get the index of the block stack in the palette.
        If it is not in the palette already it will be added first.

        :param block_stack: The block stack to get the index of.
        :return: The index of the block stack in the palette.
        """

    def count(self, value: typing.Any) -> int: ...
    def index(
        self,
        value: typing.Any,
        start: typing.SupportsInt = 0,
        stop: typing.SupportsInt = 9223372036854775807,
    ) -> int: ...
    def index_to_block_stack(
        self, arg0: typing.SupportsInt
    ) -> amulet.core.block.BlockStack:
        """
        Get the block stack at the specified palette index.

        :param index: The index to get
        :return: The block stack at that index
        :raises IndexError if there is no block stack at that index.
        """



from __future__ import annotations

import collections.abc
import types
import typing
from builtins import str as PlatformType

__all__ = [
    "PlatformType",
    "PlatformVersionContainer",
    "VersionNumber",
    "VersionRange",
    "VersionRangeContainer",
]

class PlatformVersionContainer:
    """
    A class storing platform identifier and version number.
    Thread safe.
    """

    def __init__(self, platform: str, version: VersionNumber) -> None: ...
    def __repr__(self) -> str: ...
    @property
    def platform(self) -> str:
        """
        Get the platform identifier.
        """

    @property
    def version(self) -> VersionNumber:
        """
        Get the version number.
        """

class VersionNumber:
    """
    This class is designed to store semantic versions and data versions and allow comparisons between them.
    It is a wrapper around std::vector<std::int64_t> with special comparison handling.
    The version can contain zero to max(int64) values.
    Undefined trailing values are implied zeros. 1.1 == 1.1.0
    All methods are thread safe.

    >>> v1 = VersionNumber(1, 0, 0)
    >>> v2 = VersionNumber(1, 0)
    >>> assert v2 == v1

    This class should also be used to store single number data versions.
    >>> v3 = VersionNumber(3578)
    """

    def __contains__(self, arg0: typing.SupportsInt) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: VersionNumber) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __ge__(self, arg0: VersionNumber) -> bool: ...
    @typing.overload
    def __getitem__(self, item: typing.SupportsInt) -> int: ...
    @typing.overload
    def __getitem__(self, item: slice) -> list[int]: ...
    def __gt__(self, arg0: VersionNumber) -> bool: ...
    def __hash__(self) -> int: ...
    def __init__(self, *args: typing.SupportsInt) -> None: ...
    def __iter__(self) -> collections.abc.Iterator[int]: ...
    def __le__(self, arg0: VersionNumber) -> bool: ...
    def __len__(self) -> int: ...
    def __lt__(self, arg0: VersionNumber) -> bool: ...
    def __repr__(self) -> str: ...
    def __reversed__(self) -> collections.abc.Iterator[int]: ...
    def __str__(self) -> str: ...
    def count(self, value: typing.SupportsInt) -> int: ...
    def cropped_version(self) -> list[int]:
        """
        The version number with trailing zeros cut off.
        """

    def index(
        self,
        value: typing.SupportsInt,
        start: typing.SupportsInt = 0,
        stop: typing.SupportsInt = 18446744073709551615,
    ) -> int: ...
    def padded_version(self, len: typing.SupportsInt) -> list[int]:
        """
        Get the version number cropped or padded with zeros to the given length.
        """

class VersionRange:
    """
    A class storing platform identifier and minimum and maximum version numbers.
    Thread safe.
    """

    __hash__: typing.ClassVar[None] = None  # type: ignore
    @typing.overload
    def __eq__(self, arg0: VersionRange) -> bool: ...
    @typing.overload
    def __eq__(self, arg0: typing.Any) -> bool | types.NotImplementedType: ...
    def __init__(
        self, platform: str, min_version: VersionNumber, max_version: VersionNumber
    ) -> None: ...
    def __repr__(self) -> str: ...
    def contains(self, arg0: str, arg1: VersionNumber) -> bool:
        """
        Check if the platform is equal and the version number is within the range.
        """

    @property
    def max_version(self) -> VersionNumber:
        """
        The maximum version number
        """

    @property
    def min_version(self) -> VersionNumber:
        """
        The minimum version number
        """

    @property
    def platform(self) -> str:
        """
        The platform identifier.
        """

class VersionRangeContainer:
    """
    A class that contains a version range.
    """

    def __init__(self, version_range: VersionRange) -> None: ...
    def __repr__(self) -> str: ...
    @property
    def version_range(self) -> VersionRange:
        """
        The version range.
        """
